/*
Copyright The KCP Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"context"
	"time"

	logicalcluster "github.com/kcp-dev/apimachinery/pkg/logicalcluster"

	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"

	v1alpha1 "github.com/kcp-dev/kcp/pkg/apis/tenancy/v1alpha1"
	scheme "github.com/kcp-dev/kcp/pkg/client/clientset/versioned/scheme"
)

// WorkspaceShardsGetter has a method to return a WorkspaceShardInterface.
// A group's client should implement this interface.
type WorkspaceShardsGetter interface {
	WorkspaceShards() WorkspaceShardInterface
}

// WorkspaceShardInterface has methods to work with WorkspaceShard resources.
type WorkspaceShardInterface interface {
	Create(ctx context.Context, workspaceShard *v1alpha1.WorkspaceShard, opts v1.CreateOptions) (*v1alpha1.WorkspaceShard, error)
	Update(ctx context.Context, workspaceShard *v1alpha1.WorkspaceShard, opts v1.UpdateOptions) (*v1alpha1.WorkspaceShard, error)
	UpdateStatus(ctx context.Context, workspaceShard *v1alpha1.WorkspaceShard, opts v1.UpdateOptions) (*v1alpha1.WorkspaceShard, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1alpha1.WorkspaceShard, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1alpha1.WorkspaceShardList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.WorkspaceShard, err error)
	WorkspaceShardExpansion
}

// workspaceShards implements WorkspaceShardInterface
type workspaceShards struct {
	client  rest.Interface
	cluster logicalcluster.LogicalCluster
}

// newWorkspaceShards returns a WorkspaceShards
func newWorkspaceShards(c *TenancyV1alpha1Client) *workspaceShards {
	return &workspaceShards{
		client:  c.RESTClient(),
		cluster: c.cluster,
	}
}

// Get takes name of the workspaceShard, and returns the corresponding workspaceShard object, and an error if there is any.
func (c *workspaceShards) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.WorkspaceShard, err error) {
	result = &v1alpha1.WorkspaceShard{}
	err = c.client.Get().
		Cluster(c.cluster).
		Resource("workspaceshards").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of WorkspaceShards that match those selectors.
func (c *workspaceShards) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.WorkspaceShardList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.WorkspaceShardList{}
	err = c.client.Get().
		Cluster(c.cluster).
		Resource("workspaceshards").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested workspaceShards.
func (c *workspaceShards) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Cluster(c.cluster).
		Resource("workspaceshards").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a workspaceShard and creates it.  Returns the server's representation of the workspaceShard, and an error, if there is any.
func (c *workspaceShards) Create(ctx context.Context, workspaceShard *v1alpha1.WorkspaceShard, opts v1.CreateOptions) (result *v1alpha1.WorkspaceShard, err error) {
	result = &v1alpha1.WorkspaceShard{}
	err = c.client.Post().
		Cluster(c.cluster).
		Resource("workspaceshards").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(workspaceShard).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a workspaceShard and updates it. Returns the server's representation of the workspaceShard, and an error, if there is any.
func (c *workspaceShards) Update(ctx context.Context, workspaceShard *v1alpha1.WorkspaceShard, opts v1.UpdateOptions) (result *v1alpha1.WorkspaceShard, err error) {
	result = &v1alpha1.WorkspaceShard{}
	err = c.client.Put().
		Cluster(c.cluster).
		Resource("workspaceshards").
		Name(workspaceShard.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(workspaceShard).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *workspaceShards) UpdateStatus(ctx context.Context, workspaceShard *v1alpha1.WorkspaceShard, opts v1.UpdateOptions) (result *v1alpha1.WorkspaceShard, err error) {
	result = &v1alpha1.WorkspaceShard{}
	err = c.client.Put().
		Cluster(c.cluster).
		Resource("workspaceshards").
		Name(workspaceShard.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(workspaceShard).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the workspaceShard and deletes it. Returns an error if one occurs.
func (c *workspaceShards) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Cluster(c.cluster).
		Resource("workspaceshards").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *workspaceShards) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Cluster(c.cluster).
		Resource("workspaceshards").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched workspaceShard.
func (c *workspaceShards) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.WorkspaceShard, err error) {
	result = &v1alpha1.WorkspaceShard{}
	err = c.client.Patch(pt).
		Cluster(c.cluster).
		Resource("workspaceshards").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
